use maud::{PreEscaped, html};

pub mod search;

use crate::request::{RequestContext, StringResponse};

use rocket::http::{ContentType, Status};

/// Represents the HTML structure of a page shell, including the head, body class, and body content.
///
/// This structure is used to construct the overall HTML structure of a page, making it easier to generate consistent HTML pages dynamically.
pub struct Shell {
    /// The HTML content for the `<head>` section of the page.
    head: PreEscaped<String>,
    /// An optional class attribute for the `<body>` element.
    body_class: Option<String>,
    /// The HTML content for the static body portion.
    body_content: PreEscaped<String>,
}

impl Shell {
    /// Constructs a new `Shell` instance with the given head content, body content, and body class.
    ///
    /// # Arguments
    /// * `head` - The HTML content for the page's head.
    /// * `body_content` - The HTML content for the body of the page.
    /// * `body_class` - An optional class to apply to the `<body>` element.
    ///
    /// # Returns
    /// A `Shell` instance encapsulating the provided HTML content and attributes.
    #[must_use]
    pub const fn new(
        head: PreEscaped<String>,
        body_content: PreEscaped<String>,
        body_class: Option<String>,
    ) -> Self {
        Self {
            head,
            body_class,
            body_content,
        }
    }

    /// Renders the full HTML page using the shell structure, with additional content and a title.
    ///
    /// # Arguments
    /// * `content` - The additional HTML content to render inside the main content div.
    /// * `title` - The title of the page, rendered inside the `<title>` element.
    ///
    /// # Returns
    /// A `PreEscaped<String>` containing the full HTML page content.
    #[must_use]
    pub fn render(&self, content: PreEscaped<String>, title: &str) -> PreEscaped<String> {
        html! {
                html {
                    head {
                        title { (title) };
                        (self.head)
                    };
                    @if self.body_class.is_some() {
                    body class=(self.body_class.as_ref().unwrap()) {
                        (self.body_content);

                        div id="main_content" {
                            (content)
                        };
                    };
                } @else {
                    body {
                        (self.body_content);

                        div id="main_content" {
                            (content)
                        };
                    };
                }
            }
        }
    }
}

/// Renders a full page or an HTMX-compatible fragment based on the request context.
///
/// If the request is not an HTMX request, this function uses the provided shell to generate
/// a full HTML page. If it is an HTMX request, only the provided content is rendered.
///
/// # Arguments
/// * `content` - The HTML content to render.
/// * `title` - The title of the page for full-page rendering.
/// * `ctx` - The `RequestContext` containing request metadata.
/// * `shell` - The `Shell` instance used for full-page rendering.
///
/// # Returns
/// A `StringResponse`
pub async fn render_page(
    content: PreEscaped<String>,
    title: &str,
    ctx: RequestContext,
    shell: &Shell,
) -> StringResponse {
    if ctx.is_htmx {
        (Status::Ok, (ContentType::HTML, content.into_string()))
    } else {
        (
            Status::Ok,
            (
                ContentType::HTML,
                shell.render(content, title).into_string(),
            ),
        )
    }
}

/// Generates an HTML link with HTMX attributes for dynamic behavior.
///
/// This function creates an `<a>` element with attributes that enable HTMX behavior for navigation without reload.
///
/// # Arguments
/// * `url` - The URL to link to.
/// * `class` - The CSS class for styling the link.
/// * `onclick` - The JavaScript `onclick` handler for the link.
/// * `content` - The content inside the link element.
///
/// # Returns
/// A `PreEscaped<String>` containing the rendered HTML link element.
#[must_use]
pub fn htmx_link(
    url: &str,
    class: &str,
    onclick: &str,
    content: PreEscaped<String>,
) -> PreEscaped<String> {
    html!(
        a class=(class) onclick=(onclick) href=(url) hx-get=(url) hx-target="#main_content" hx-push-url="true" hx-swap="innerHTML" {
            (content);
        };
    )
}

/// Generates a `<script>` element containing the provided JavaScript code.
///
/// This function wraps the provided JavaScript code in a `<script>` tag,
/// allowing for easy inclusion of custom scripts in the rendered HTML.
///
/// # Arguments
/// * `script` - The JavaScript code to include.
///
/// # Returns
/// A `PreEscaped<String>` containing the rendered `<script>` element.
#[must_use]
pub fn script(script: &str) -> PreEscaped<String> {
    html!(
        script {
            (PreEscaped(script))
        };
    )
}
